/******************************************************************************
* FLOPPY_RDY()
*   Bill Johnson, Sunrise Software, Baltimore, Md (CIS 73577,465)
*   adapted from code by Tom Crosley, SOFTWEST, Sunnyvale, CA (CIS 70205,533)
*   July 21, 1989
*
*   Checks to see if floppy disk drive is ready. Use before doing any disk
*       I/O operation to avoid hanging the machine or getting untidy error
*       messages. Default drive is A: is parameter is omitted.
*
*   result = FLOPPY_RDY([drive_code])
*
*       result      -   numeric.
*                       values are:
*                          3 - diskette write protected
*                          6 - diskette removed or drive door open
*                        128 - timeout
*       drive_code  -   numeric.
*                       values are:
*                          0 - A:
*                          1 - B:
*
* Compiled using Micrsoft 5.1, but should be adaptable to any library
* that allows calls to BIOS Int routines.
*
* sample code for calling floppy_rdy()
*
*    PARAMETER drive
*    CLEAR
*    IF TYPE("drive") != "U"
*       IF UPPER(drive) = "B"
*          drv = 1  && DRIVE B:
*        ELSE
*          drv = 0  && DRIVE A:
*       ENDIF
*     ELSE
*       drv = 0  && DEFAULT TO A:
*    ENDIF
*
*    result = floppy_rdy(drv)  && is disk ready?
*
*    IF result = 0
*       @ 24,0  SAY 'Disk Drive '
*       @ 24,11 SAY IIF(drv = 1, "A:", "B:")
*       @ 24,13 SAY 'is ready!'
*     ELSE
*       DO CASE
*          CASE result = 3
*             err_msg = "Diskette Is Write-Protected"
*          CASE result = 6 .OR. result = 128
*             err_msg = "Diskette Drive Door Is Open"
*          OTHERWISE
*             err_msg = "Diskette Is Unformatted or Wrong Density"
*       ENDCASE
*       @ 24,0 SAY err_msg
*    ENDIF
*
*******************************************************************************/

#include <nandef.h>
#include <extend.h>
#include <stdio.h>
#include <ctype.h>
#include <dos.h>
#include <malloc.h>

#define SECTOR_SIZE 512     /* default sector size */
#define RETRY       1       /* # of retries */

/* check if a floppy drive is ready using Int 13H Funtion 04 (Verify Sector) */
/* returns TRUE if drive ready, FALSE if not (e.g. drive door open)          */

CLIPPER floppy_rdy()
{
    int      cnt, drv;
    union    REGS inregs, outregs;
    struct   SREGS segregs;
    void far *buffer;

    if (PCOUNT == 1 && ISNUM(1))   {
       drv = _parni(1);                     /* specified drive               */
       if (drv < 0 || drv > 1) drv = 0;     /* if specified drive not a or b */
                                            /*   make it a:                  */
    } else {
       drv = 0;                             /* default drive - A: */
    }

    buffer = malloc(SECTOR_SIZE);           /* allocate temp sector buffer */
    cnt = 0;

    while (1) {

        /* read sector 1, side 0, cylinder 0, to test for good disk */
        inregs.h.ah = 0x02;                 /* function 2 - read */
        inregs.h.al = 1;                    /* # of sectors */
        inregs.h.ch = 0;                    /* 1st cylinder */
        inregs.h.cl = 1;                    /* 1st sector */
        inregs.h.dh = 0;                    /* 1st side */
        inregs.h.dl = drv;                  /* drive ('A' = 0) */
        segregs.es = FP_SEG(buffer);        /* ES:BX pt to buffer (for old BIOS's) */
        inregs.x.bx = FP_OFF(buffer);
        int86x(0x13, &inregs, &outregs, &segregs);      /* call Int 13H */

        if (!outregs.h.ah) {                /* read worked ok */

                                            /* try to write it back out */
           inregs.h.ah = 0x03;              /* function 3 - write */
           inregs.h.al = 1;                 /* # of sectors */
           inregs.h.ch = 0;                 /* 1st cylinder */
           inregs.h.cl = 1;                 /* 1st sector */
           inregs.h.dh = 0;                 /* 1st side */
           inregs.h.dl = drv;               /* drive ('A' = 0) */
           segregs.es = FP_SEG(buffer);     /* ES:BX pt to buffer (for old BIOS's) */
           inregs.x.bx = FP_OFF(buffer);
           int86x(0x13, &inregs, &outregs, &segregs);  /* call Int 13H */
        }

        if (!outregs.h.ah) break;           /* status ok -- return */
        if (++cnt > RETRY) break;           /* already retried */

                                            /* reset drive before checking again */
        inregs.h.ah = 0x00;                 /* function # */
        inregs.h.dl = drv;                  /* drive ('A' = 0) */
        int86(0x13, &inregs, &outregs);     /* call Int 13H */

    }
    free(buffer);
    _retni((int)outregs.h.ah);
}

