/****************************************************************************
*  SHADOW.C         Clipper UDF for realistic drop shadows                  *
*                                                                           *
*  By: Scott McIntosh                                                       *
*  Placed in Public Domain 1989                                             *
*                                                                           *
*  Date: 8/19/1989                                                          *
*  Version: 1.0                                                             *
*                                                                           *
*  Syntax:                                                                  *
*  SHADOW( <expN1>, <expN2>, <expN3>, <expN4>, [<expN5>], [<expN6>] )       *
*                                                                           *
*  Where: N1 through N4 are t,b,l,r coordinates of the Box to               *
*             be shadowed and N5/N6 is foreground/background values         *
*                                                                           *
*  Color parameter values:                                                  *
*          Black        =  0    Foreground/Background                       *       *
*          Blue         =  1    Foreground/Background                       *
*          Green        =  2    Foreground/Background                       *
*          Cyan         =  3    Foreground/Background                       *
*          Red          =  4    Foreground/Background                       *
*          Magenta      =  5    Foreground/Background                       *
*          Brown        =  6    Foreground/Background                       *
*          Lt. Gray     =  7    Foreground/Background                       *
*          Dk. Gray     =  8    Foreground Only                             *
*          Lt. Blue     =  9    Foreground Only                             *
*          Lt. Green    = 10    Foreground Only                             *
*          Lt. Cyan     = 11    Foreground Only                             *
*          Lt. Red      = 12    Foreground Only                             *
*          Lt. Magenta  = 13    Foreground Only                             *
*          Yellow       = 14    Foreground Only                             *
*          White        = 15    Foreground Only                             *
*                                                                           *
*                                                                           *
*  Example:  @ 10,10 CLEAR TO 20,20                                         *
*            @ 10,10 TO 20,20 DOUBLE                                        *
*            SHADOW( 10, 10, 20, 20 )                                       *
*              - or -                                                       *
*            SHADOW( 10, 10, 20, 20, 7, 8 )  && Lt. Gray / Dk. Gray         *
*                                                                           *
*  Returns: Nothing                                                         *
*                                                                           *
*  Compile: CL /AL /Gs /Zi /Zl /FPa /c                                      *
*                                                                           *
*                                                                           *
****************************************************************************/     

#define IS_CLIPPER           /* Remove if called from C instead of Clipper */
#define MK_FP(seg,ofs)	((void far *) \  
			   (((unsigned long)(seg) << 16) | (unsigned)(ofs)))

#include <stdio.h>
#include <dos.h>

#ifdef IS_CLIPPER
#include "nandef.h" 
#include "extend.h"
#endif

void set_scr_attr( int top, int left, int bottom, int right, char fg, char bg);
char far * get_video_zero( void );

#ifdef IS_CLIPPER
CLIPPER SHADOW()
{
   int t, b, l, r;                       /* Coordinates of box */
   int _pcount;                          /* PCOUNT value */
   int isnum1, isnum2, isnum3, isnum4;   /* ISNUM() values */
   int par_temp;                         /* temp for _parni() value */     
   char f_gnd, b_gnd;                    /* Foreground/Background */

   _pcount = PCOUNT;                     /* Assign PCOUNT */
   isnum1 = ISNUM(1);                    /* Assign ISNUM()'s */
   isnum2 = ISNUM(2);
   isnum3 = ISNUM(3);
   isnum4 = ISNUM(4);
   f_gnd = 8;                            /* Gray default */
   b_gnd = 0;                            /* Black default */

   if( ( _pcount == 6 && isnum1 && isnum2 && isnum3 && isnum4 &&
                         ISNUM(5) && ISNUM(6) )
    || ( _pcount == 4 && isnum1 && isnum2 && isnum3 && isnum4 )
   {

      if( _pcount == 6 )
      {
         par_temp = _parni(5);
         if( ( par_temp > 0 ) && ( par_temp < 16 ) ) /* check foreground */
            f_gnd = par_temp;

         par_temp = _parni(6);
         if( ( par_temp > 0 ) && ( par_temp < 8 ) )  /* check background */
            b_gnd = par_temp;
      }

      t = _parni(1);
      l = _parni(2);
      b = _parni(3);
      r = _parni(4);

      /* Check for valid coordinates */
      if( !( t<0 || t>24 || b<0 || b>24 || l<0 || l>79 || r<0 || r>79 ) )
      {
         if( t<b && r>l )      /* Check for valid coordinate relationships */                      
         {
            /* Shadow on right side of box */
            set_scr_attr( t+1, r+1,  b , r+2, f_gnd, b_gnd );

            /* Shadow on bottom of box */
            set_scr_attr( b+1, l+2, b+1, r+2, f_gnd, b_gnd );
         }
      }
   }
   _ret();
}
#endif


/*-----------------------------------------------------------------------*/
void set_scr_attr( int top, int left, int bottom, int right, char fg, char bg)
{
   char far * video_zero;                   /* Video 0,0 Reference */
   char far * ptr;                          /* Pointer to attribute byte */
   char color;                              /* Foreground/Background Pair */
   int row;                                 /* Current screen row */
   int col;                                 /* Current screen col */
                                                     
   video_zero = get_video_zero();
   if( video_zero )
   {
      color = ( bg << 4 ) | fg;               /* Set attribute byte value */
      if( right-left >= bottom-top )          /* Determine greatest Aspect */
      {
         for( row = top; row <= bottom; row++ )    /* Horiz aspect greater */
         {
            col = left;                        /* Reset col to left border */
            ptr = (video_zero + 1) + (row * 160) + (left * 2);

            for( ; col <= right; col++ )
            {
               *ptr = color;                /* Set attribute */
               ptr++;
               ptr++;                       /* Next attribute byte */
               /* Note: Incrementes pointer twice (instead of pointer + 2 )
                  for speed. */  
            }
         }
      }
      else
      {
         for( col = left; col <= right; col++ ) /* Vert aspect greater */
         {
            row = top;                      /* Reset row to top */
            ptr = (video_zero + 1) + (row * 160) + (col * 2);

            for( ; row <= bottom; row++ )
            {
               *ptr = color;
               ptr += 160;
            }
         }
      }
   }
   return;
}


char far * get_video_zero( void )   
{
   char far * v_zero;                                /* return pointer */
   union REGS in_reg, out_reg;                  

   in_reg.h.ah = 0x0F;                           /* request service 0F */
   int86( 0x10, &in_reg, &out_reg );             /* generate interrupt */

   if( out_reg.h.al == 0x07 )                   /* video is monochrome */
      v_zero = MK_FP( 0xB000, 0 );
   else     
   {
      /* video is color: must calculate page offset (BH x 1000h) */
      v_zero = MK_FP( ( 0xB800 + ( out_reg.h.bh * 0x1000 ) ), 0 );
   }
   return( v_zero );
}

/*---------- EOF: SHADOW.C ------------------------------------------------*/
  
  
